/*
 *  typespeed - measures your typing speed
 *  Copyright (C) 1999-2003   Jani Ollikainen  <bestis@iki.fi>
 *                          & Jaakko Manelius  <jman@iki.fi>
 *  Copyright (C) 2006   Tobias Stoeckmann  <tobias@bugol.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * menu.c - menu & misc drawing functions
 */

#include <curses.h>
#include <stdlib.h>

#include "typespeed.h"
#include "version.h"

extern int	 choose_wordfile(int);
extern void	 close_network(void);
extern int	 init_network(char *, int);
extern int	 level(int);
extern void	 liima_mvgetnstr(int, int, char *, int);
extern void	 multipmenu(void);
extern void	*xmalloc(size_t);
extern void	 xcolor_set(short);

void	drawmenu(void);
void	drawscreen(void);
void	multipmenu(void);
void	optionmenu(void);
void	showhighscores(void);
void	tell_story(void);
void	pressanykey(int, int);

extern int	 misses;
extern char	*rankki[11];
extern int	 typesocket;

/* Draws main menu onto screen. */
void
drawmenu(void)
{
	drawscreen();
	xcolor_set(5);

	mvprintw( 3, 30, "Typespeed %s", TVERSION);

	mvaddstr( 5, 30, "1. Test Your Speed");
	mvaddstr( 6, 30, "2. Network Head2Head");
	mvaddstr( 7, 30, "3. Story/Credits/RTFM!");
	mvaddstr( 8, 30, "4. Show Highscores");
	mvaddstr( 9, 30, "5. Options");
	mvaddstr(10, 30, "6. Quit");

	mvaddstr(12, 30, "Choose: ");
}

/* Draws credits onto screen. */
void
tell_story(void)
{
	drawscreen();
	xcolor_set(5);

	mvprintw(3, 2, "Typespeed %s", TVERSION);
	mvaddstr(5, 2, "Typespeed is made by  Jani Ollikainen  <bestis@iki.fi>");
	mvaddstr(6, 2, "                    & Jaakko Manelius  <jman@iki.fi>");
	mvaddstr(7, 2, "Current maintainer is Tobias Stoeckmann  <tobias@bugol.de>");

	mvaddstr(9, 2, "Typespeed's idea is ripped from ztspeed (a dos game made");
	mvaddstr(10, 2, "by Zorlim). Typespeed doesn't give scores that can be");
	mvaddstr(11, 2, "compared with ztspeed anymore. We wanted to use our way.");
	mvaddstr(13, 2, "Idea of the game should be clear to anyone, just type and");
	mvaddstr(14, 2, "type it fast, or be a lewser.");
	mvaddstr(16, 2, "Bugs/Ideas/Comments to <tobias@bugol.de>");

	nodelay(stdscr, 0);
	cbreak();
	flushinp();

	pressanykey(18, 2);
}

/* Switches "number" from 1 to 0 or any other value to 1. */
int
dochange(int number)
{
	return (number == 1) ? 0 : 1;
}

/* Draws multiplayer menu. */
void
multipmenu(void)
{
	char *servo;
	int exitnow, wheretogo;

	exitnow = 0;
	servo = xmalloc(80);

	do {
		drawscreen();

		xcolor_set(5);

		mvprintw(3, 30, "Typespeed %s", TVERSION);

		mvaddstr(5, 30, "1. Server");
		mvaddstr(6, 30, "2. Client");
		mvaddstr(7, 30, "3. Port");
		mvaddstr(8, 30, "4. Return");

		if (opt.server) {
			xcolor_set(3);
			mvaddstr(5, 44, "ON");
		} else {
			xcolor_set(6);
			mvaddstr(5, 44, "OFF");
		}
		if (opt.client) {
			xcolor_set(3);
			mvaddstr(6, 44, "ON");
		} else {
			xcolor_set(6);
			mvaddstr(6, 44, "OFF");
		}

		xcolor_set(5);

		mvprintw(7, 44, "%d", opt.port);

		mvaddstr(10, 30, "Choose: ");

		nodelay(stdscr, 0);
		cbreak();

		while ((wheretogo = getch()) == ERR);

		switch (wheretogo) {
		case '1':
			if (!opt.client) {
				if (!opt.server) {
					opt.server = 1;
					exitnow = 1;
				} else {
					close_network();
					opt.server = 0;
					typesocket = 0;
					exitnow = 0;
				}
			}
			break;
		case '2':
			if (!opt.server) {
				if (!opt.client) {
					opt.client = 1;
					mvaddstr(10, 30, "Enter Host: ");
					liima_mvgetnstr(10, 42, servo, 80);
					exitnow = 1;
				} else {
					opt.client = 0;
					typesocket = 0;
					exitnow = 0;
				}
			}		
			break;
		case '3':
			mvaddstr(10, 30, "Enter Port: ");
			liima_mvgetnstr(10, 42, servo, 80);
			opt.port = strtol(servo, NULL, 10);
			if (opt.port <= 1024)
				opt.port = 6025;
			exitnow = 0;
			break;
		case '4':
			exitnow = 1;
			break;
		default:
			break;
		}
	} while (!exitnow);

	if (wheretogo == 52 || (!opt.client && !opt.server))
		return;

	drawscreen();

	xcolor_set(5);
	mvprintw(3, 30, "Typespeed %s", TVERSION);
	mvaddstr(5, 30, "When you get connect, choose a wordfile and");
	mvaddstr(6, 30, "then wait for the game start. The game starts as");
	mvaddstr(7, 30, "soon as the other player has chosen a wordfile.");

	refresh();

	if (opt.server || opt.client)
		do {
			typesocket = init_network(servo, 1);
		} while (typesocket < 0);

	flushinp();
	free(servo);
}

/* Draws option menu. */
void
optionmenu(void)
{
	char *trasp;
	int wheretogo;

	trasp = xmalloc(4);

	do {
		drawscreen();

		xcolor_set(5);
		mvprintw(3, 30, "Typespeed %s", TVERSION);

		mvaddstr(5, 30, "1. Colors");
		mvaddstr(6, 30, "2. Cheat");
		mvaddstr(7, 30, "3. Training");
		mvaddstr(8, 30, "4. Trng Speed");
		mvaddstr(9, 30, "5. Reset a Scorefile");
		mvaddstr(10, 30, "6. Return");
		if (opt.usecolors) {
			xcolor_set(3);
			mvaddstr(5, 44, "ON");
		} else {
			xcolor_set(6);
			mvaddstr(5, 44, "OFF");
		}
		if (opt.cheat) {
			xcolor_set(3);
			mvaddstr(6, 44, "ON");
		} else {
			xcolor_set(6);
			mvaddstr(6, 44, "OFF");
		}
		if (opt.training) {
			xcolor_set(3);
			mvaddstr(7, 44, "ON");
		} else {
			xcolor_set(6);
			mvaddstr(7, 44, "OFF");
		}
		xcolor_set(3);
		mvprintw(8, 44, "%d", opt.traspeed);

		xcolor_set(5);
		mvaddstr(12, 30, "Choose: ");

		nodelay(stdscr, 0);
		cbreak();

		while ((wheretogo = getch()) == ERR);

		switch (wheretogo) {
		case '5':
			choose_wordfile(2);
			break;
		case '4':
			mvaddstr(12, 30, "Enter Speed:    ");
			liima_mvgetnstr(12, 43, trasp, 2);
			if ((strtol(trasp, NULL, 10) >= 2) && (10 >= strtol(trasp, NULL, 10))) {
				opt.traspeed = strtol(trasp, NULL, 10);
			}
			break;
		case '3':
			if (!opt.client && !opt.server)
				opt.training = dochange(opt.training);
			break;
		case '2':
			opt.cheat = dochange(opt.cheat);
			break;
		case '1':
			xcolor_set(2);
			opt.usecolors = dochange(opt.usecolors);
			break;
		default:
			break;
		}
	} while (wheretogo != '6');

	free(trasp);
}

/* Shows highscores. */
void
showhighscores(void)
{
	choose_wordfile(1);
}

/*
 * Prints status values - no descriptions though!
 * Cursor will be put into "inputpos" (y = 21, x = 2 + inputpos).
 */
void
drawstatus(int inputpos)
{
	mvprintw(23, 29, "%-9s", rankki[level(now.score)]);

	xcolor_set(3);
	mvprintw(23, 45, "%d", now.score);
	mvprintw(23, 55, "%2.2f", now.totalspeed);
	mvprintw(23, 66, "%2.2f", now.speed);
	mvprintw(23, 78, "%2d", misses);

	if (inputpos > -1)
		move(23, 2 + inputpos);
}

/* Clears screen and draws status bar. */
void
drawscreen(void)
{
	clear();

	xcolor_set(4);
	move(22, 0);
	hline(ACS_HLINE, 80);
	mvaddch(23, 1, '>');
	mvaddch(23, 21, '<');

	xcolor_set(2);
	if (opt.client)
		mvaddstr(22, 70, "CLI");
	else if (opt.server)
		mvaddstr(22, 70, "SER");	
	else if (opt.training)
		mvaddstr(22, 76, "TRA");
	if (opt.cheat)
		mvaddstr(22, 73, "CHE");

	xcolor_set(5);
	mvaddstr(23, 23, "Rank:");
	mvaddstr(23, 38, "Score:");
	mvaddstr(23, 49, "tCPS:");
	mvaddstr(23, 60, "cCPS:");
	mvaddstr(23, 71, "Misses:");

	drawstatus(0);
}

void
pressanykey(int y, int x)
{
	xcolor_set(4);
	mvaddstr(y, x, "Press any key to continue...");
	while (getch() == ERR);
}

