/*
 *  typespeed - measures your typing speed
 *  Copyright (C) 1999-2003   Jani Ollikainen  <bestis@iki.fi>
 *                          & Jaakko Manelius  <jman@iki.fi>
 *  Copyright (C) 2006   Tobias Stoeckmann  <tobias@bugol.de>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * network.c - network  for typespeed
 *
 * thanks goto Steinar H. Gunderson <sgunderson@bigfoot.com>
 * for the basic functions which we got from him
 */

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>

#include <netinet/in.h>
#include <arpa/inet.h>

#include <curses.h>
#include <err.h>
#include <errno.h>
#include <limits.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "typespeed.h"
#include "version.h"

extern int	ttss;

extern void	 endcursestuff(void);
extern void	*xmalloc(size_t);

int net_ident(int, int, int);

void
close_network(void)
{
	close(ttss);
	ttss = 0;
}

int
init_network(char *serv, int graph)
{
	struct sockaddr_in addr;
	int i, one, sock, ss;

	one = 1;

	sock = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

	if (sock == -1) {
		if (graph)
			endcursestuff();
		err(1, "socket()");
	}
	if (opt.client) {
		addr.sin_family = AF_INET;
		addr.sin_addr.s_addr = inet_addr(serv);
		addr.sin_port = htons(opt.port);

		if (addr.sin_addr.s_addr == INADDR_NONE) {
			struct hostent *he;

			if (graph) {
				mvprintw(10,30,"Looking up %s...",serv);
				refresh();
			} else
				printf("Looking up %s...\n", serv);
			he = gethostbyname(serv);

			if (he == NULL) {
				if (graph)
					endcursestuff();
				err(1, "%s", serv);
			}
			bcopy(he->h_addr, (char *)&addr.sin_addr, he->h_length);
		}

		if (graph) {
			mvprintw(10,30,"Connecting to typespeed server at %s (port %d)...\n", inet_ntoa(addr.sin_addr), opt.port);
			refresh();
		} else
			printf("Connecting to typespeed server at %s (port %d)...\n", inet_ntoa(addr.sin_addr),opt.port);

		if (connect(sock, (struct sockaddr *)&addr, sizeof(addr)) == -1) {
			if (graph)
				endcursestuff();
			err(1, "%s", serv);
		}

		i = net_ident(sock, 0, graph);
		if (i)
			return i;
	} else {
		bzero((char *)&addr, sizeof(addr));

		addr.sin_family = AF_INET;
		addr.sin_addr.s_addr = INADDR_ANY;
		addr.sin_port = htons(opt.port);

		ss = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
		if (ss == -1) {
			if (graph)
				endcursestuff();
			err(1, "socket()");
		}
		setsockopt(ss, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(one));
		if (bind(ss, (struct sockaddr *) &addr, sizeof(struct sockaddr)) == -1) {
			if (graph)
				endcursestuff();
			err(1, "bind()");
		}
	
		if (listen(ss, 1) == -1) {
			if (graph)
				endcursestuff();
			err(1, "listen()");
		}
	
		if (graph) {
			mvaddstr(10, 30, "Waiting for connections...");
			refresh();
		} else
			puts("Waiting for connections...");

		if ((sock = accept(ss, INADDR_ANY, NULL)) == -1) {
			if (graph)
				endcursestuff();
			err(1, "accept()");
		}

		ttss = ss;

		i = net_ident(sock, ss, graph);
		if (i) {
			close(ss);
			return i;
		}
	}
	ioctl(sock, FIONBIO, &one);

	return sock;
}

int
net_ident(int typesock, int tss, int graph)
{
	char buf[60];
	int i;

	sprintf(buf,"TYPESPEED %s\n", TVERSION);
	send(typesock, buf, strlen(buf), 0);

	strcpy(buf, "                                                           ");

	if (graph) {
		mvaddstr(10, 30, "Waiting for other party to identify...");
		refresh();
	} else {
		puts("Connect.");
		puts("Waiting for other party to identify...");
	}

	i = recv(typesock, buf, 59, MSG_PEEK);
	if (i == -1 && errno != EWOULDBLOCK) {
		if (graph)
			endcursestuff();
		err(1, "recv()");
	}
	if (i > 0) {
		recv(typesock, buf, strlen(buf) + 1, 0);
		if (!strncmp(buf, "TYPESPEED", 9)) {
			if (graph) {
				mvprintw(10, 30, "Remote is using: %s", buf);
				refresh();
			} else
				printf("Remote is using: %s\n",buf);
		} else {
			if (graph) {
				mvaddstr(10, 30, "Remote is not TYPESPEED");
				refresh();
			} else
				puts("Remote is not TYPESPEED");
			close(tss);
			return -69;
		}
	} else
		return -69;
	return 0;
}

void
net_waitgame(int typesock)
{
	char buf[60];
	int i;

	sprintf(buf, "TYPESPEED %s\n", TVERSION);

	i = send(typesock, buf, strlen(buf), 0);	

	strcpy(buf, "                                                           ");

	for (;;) {
		i = recv(typesock, buf, 59, MSG_PEEK);
		if (i == -1 && errno != EWOULDBLOCK) {
			endcursestuff();
			err(1, "recv()");
		}
		if (i > 0) {
			recv(typesock, buf, strlen(buf) + 1, 0);
                	if (!strncmp(buf, "TYPESPEED", 9))
				break;
		}
	}
}

void
net_swapscore(int typesock, struct stats *stat, struct stats *stat2)
{
	char buf[60];
	int i;
	char *temp,*temp2;
	char *number;
	size_t x;

	sprintf(buf, "SCORE: %d %f %f %f\n", stat->score, stat->speed, stat->totalspeed, stat->ratio);

	i = send(typesock, buf, strlen(buf), 0);

	strcpy(buf, "                                                           ");

        for (;;) {
		i = recv(typesock, buf, 59, MSG_PEEK);
		if (i == -1 && errno != EWOULDBLOCK) {
			endcursestuff();
			err(1, "recv()");
		}
		if (i > 0) {
			recv(typesock, buf, strlen(buf) + 1, 0);
		        if (!strncmp(buf, "SCORE:", 6))
				break;
		}
	}

	number = xmalloc(80);
	temp = xmalloc(80);

	for (i = 0, x = 7; x <= strlen(buf); x++) {
		temp[i] = buf[x];
		i++;
	}

	strncpy(number, temp, strcspn(temp, " "));
	stat2->score = strtol(number, NULL, 10);
	free(number);
	temp2 = xmalloc(80);

	for (i = 0, x = strcspn(temp, " ") + 1; x <= strlen(temp); x++) {
		temp2[i] = temp[x];
		i++;
	}
	free(temp);

	number = xmalloc(80);
	strncpy(number, temp2, strcspn(temp2, " "));
	stat2->speed = (float)atof(number);
	free(number);
	temp = xmalloc(80);

	for (i = 0, x = strcspn(temp2, " ") + 1; x <= strlen(temp2); x++) {
		temp[i] = temp2[x];
		i++;
	}
	free(temp2);

	number = xmalloc(80);
	strncpy(number, temp, strcspn(temp, " "));
	stat2->totalspeed = (float)atof(number);
	temp2 = xmalloc(80);

	for (i = 0, x = strcspn(temp, " ") + 1;x <= strlen(temp); x++) {
		temp2[i] = temp[x];
		i++;
	}

	stat2->ratio = (float)atof(temp2);
	free(temp);
	free(temp2);
}

